import pytest
from rdflib.plugins.stores.sparqlstore import SPARQLStore

from test_utils import (
    add_data,
    can_use_gsp_read,
    can_use_gsp_write,
    can_use_sparql_query,
    can_use_sparql_update,
    cannot_use_sparql_update,
    cannot_use_gsp_write,
)

DATASET_NAME = "dalia"


class TestDaliaDatasetRead:
    @pytest.mark.parametrize("endpoint_name", ["", "query", "sparql"])
    def test_can_use_sparql_query(self, fuseki_url, endpoint_name):
        add_data(f"{fuseki_url}/{DATASET_NAME}/update")

        assert can_use_sparql_query(fuseki_url, DATASET_NAME, endpoint_name)

    @pytest.mark.parametrize("endpoint_name", ["", "get"])
    def test_can_use_gsp_get(self, fuseki_url, endpoint_name):
        add_data(f"{fuseki_url}/{DATASET_NAME}/update")

        assert can_use_gsp_read(fuseki_url, DATASET_NAME, endpoint_name)

    @pytest.mark.parametrize("endpoint_name", ["query", "sparql", "get", "data"])
    def test_endpoints_are_not_writeable_with_sparql_update(self, fuseki_url, endpoint_name):
        assert cannot_use_sparql_update(fuseki_url, DATASET_NAME, endpoint_name)

    @pytest.mark.parametrize("endpoint_name", ["query", "sparql", "get", "update"])
    def test_endpoints_are_not_writeable_with_gsp(self, fuseki_url, endpoint_name):
        assert cannot_use_gsp_write(fuseki_url, DATASET_NAME, endpoint_name)

    def test_text_search_on_propList_learningResourceTexts_returns_results_from_all_indexed_properties(self, fuseki_url):
        endpoint = f"{fuseki_url}/{DATASET_NAME}"
        store = SPARQLStore(query_endpoint=endpoint)

        result = store.query("""
            PREFIX dt: <http://dalia.education/text#>
            PREFIX text: <http://jena.apache.org/text#>

            SELECT DISTINCT ?prop
            WHERE {
                {
                    (?s ?score ?literal ?g ?prop) text:query (dt:learningResourceTexts 'Frank') .
                }
                UNION {
                    (?s ?score ?literal ?g ?prop) text:query (dt:learningResourceTexts 'GHGA') .
                }
            }
        """)
        assert set(map(lambda item: str(item.prop), list(result))) == {
            "http://purl.org/dc/terms/title",
            "http://purl.org/dc/terms/description",
            "https://schema.org/keywords",
            "https://schema.org/name",
            "https://schema.org/familyName",
            "https://schema.org/givenName",
        }

    def test_text_search_on_propList_communityTexts_returns_results_from_all_indexed_properties(self, fuseki_url):
        endpoint = f"{fuseki_url}/{DATASET_NAME}"
        store = SPARQLStore(query_endpoint=endpoint)

        result = store.query("""
            PREFIX dt: <http://dalia.education/text#>
            PREFIX text: <http://jena.apache.org/text#>

            SELECT DISTINCT ?prop
            WHERE {
                (?s ?score ?literal ?g ?prop) text:query (dt:communityTexts 'GHGA') .
            }
        """)
        assert set(map(lambda item: str(item.prop), list(result))) == {
            "http://purl.org/dc/terms/title",
            "http://purl.org/dc/terms/description",
        }

    def test_text_search_on_dcterms_title(self, fuseki_url):
        endpoint = f"{fuseki_url}/{DATASET_NAME}"
        store = SPARQLStore(query_endpoint=endpoint)

        result = store.query("""
            PREFIX dcterms: <http://purl.org/dc/terms/>
            PREFIX text: <http://jena.apache.org/text#>

            SELECT ?literal ?type
            WHERE {
                (?s ?score ?literal) text:query (dcterms:title 'NFDI4Biodiversity') .
                ?s a ?type .
            }
            ORDER BY DESC(?score)
        """)
        assert list(map(lambda item: (str(item.literal), str(item.type)), list(result))) == [
            ("NFDI4Biodiversity",
             "https://purl.org/ontology/modalia#Community"),
            ("NFDI4Biodiversity Self-Study Unit - Research Data Management for Biodiversity Data",
             "https://github.com/tibonto/educor#EducationalResource"),
        ]

    def test_text_search_on_dcterms_description(self, fuseki_url):
        endpoint = f"{fuseki_url}/{DATASET_NAME}"
        store = SPARQLStore(query_endpoint=endpoint)

        result = store.query("""
            PREFIX dcterms: <http://purl.org/dc/terms/>
            PREFIX text: <http://jena.apache.org/text#>

            SELECT ?literal
            WHERE {
                (?s ?score ?literal) text:query (dcterms:description 'BioImage Dresden') .
            }
            ORDER BY DESC(?score) LIMIT 1
        """)
        assert list(map(lambda item: str(item.literal), list(result))) == [
            "Presentation given at PoL BioImage Analysis Symposium Dresden 2023",
        ]

    def test_text_search_on_schema_keywords(self, fuseki_url):
        endpoint = f"{fuseki_url}/{DATASET_NAME}"
        store = SPARQLStore(query_endpoint=endpoint)

        result = store.query("""
            PREFIX schema: <https://schema.org/>
            PREFIX text: <http://jena.apache.org/text#>

            SELECT DISTINCT ?literal
            WHERE {
                (?s ?score ?literal) text:query (schema:keywords 'Micro*') .
            }
            ORDER BY DESC(?score)
        """)
        assert list(map(lambda item: str(item.literal), list(result))) == [
            "Microbiology",
            "Microscopy",
            "microscopy",
        ]

    def test_text_search_on_schema_name(self, fuseki_url):
        endpoint = f"{fuseki_url}/{DATASET_NAME}"
        store = SPARQLStore(query_endpoint=endpoint)

        result = store.query("""
            PREFIX schema: <https://schema.org/>
            PREFIX text: <http://jena.apache.org/text#>

            SELECT DISTINCT ?literal
            WHERE {
                (?s ?score ?literal) text:query (schema:name 'Karlsruhe') .
            }
            ORDER BY DESC(?score)
        """)
        assert list(map(lambda item: str(item.literal), list(result))) == [
            "ISE FIZ Karlsruhe",
            "FIZ Karlsruhe Video-Bits",
        ]

    def test_text_search_on_schema_familyName(self, fuseki_url):
        endpoint = f"{fuseki_url}/{DATASET_NAME}"
        store = SPARQLStore(query_endpoint=endpoint)

        result = store.query("""
            PREFIX schema: <https://schema.org/>
            PREFIX text: <http://jena.apache.org/text#>

            SELECT DISTINCT ?literal
            WHERE {
                (?s ?score ?literal) text:query (schema:familyName '*mann') .
            }
        """)
        assert "Neumann" in set(map(lambda item: str(item.literal), list(result)))

    def test_text_search_on_schema_givenName(self, fuseki_url):
        endpoint = f"{fuseki_url}/{DATASET_NAME}"
        store = SPARQLStore(query_endpoint=endpoint)

        result = store.query("""
            PREFIX schema: <https://schema.org/>
            PREFIX text: <http://jena.apache.org/text#>

            SELECT DISTINCT ?literal
            WHERE {
                (?s ?score ?literal) text:query (schema:givenName '*anja') .
            }
            ORDER BY ?literal
        """)
        assert list(map(lambda item: str(item.literal), list(result))) == [
            "Manja",
            "Tanja",
        ]


@pytest.mark.parametrize("endpoint_name", ["", "update"])
def test_endpoints_are_writeable_with_sparql_update(fuseki_url, endpoint_name):
    assert can_use_sparql_update(fuseki_url, DATASET_NAME, endpoint_name)


@pytest.mark.parametrize("endpoint_name", ["", "data"])
def test_endpoints_are_writeable_with_gsp(fuseki_url, endpoint_name):
    assert can_use_gsp_write(fuseki_url, DATASET_NAME, endpoint_name)
